﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Windows.Controls;
using Infragistics.Olap;
using Infragistics.Olap.FlatData;

namespace CustomAggregatorSample
{
    public partial class MainPage : UserControl
    {
        private ObservableCollection<UserData> _itemsList;

        public MainPage()
        {
            InitializeComponent();

            this.InitData();
            this.InitializePivotGrid();
        }

        private void InitData()
        {
            this._itemsList = new ObservableCollection<UserData>();

            UserData user1 = new UserData
                                 {
                                     FirstName = "Name1",
                                     LastName = "Family1",
                                     Age = 24,
                                     Location = "Los Angeles/USA",
                                     Rating = "AAA"
                                 };

            UserData user2 = new UserData
            {
                FirstName = "Name2",
                LastName = "Family2",
                Age = 27,
                Location = "New York/USA",
                Rating = "CCC"
            };

            UserData user3 = new UserData
            {
                FirstName = "Name3",
                LastName = "Family3",
                Age = 34,
                Location = "Ottawa/Canada",
                Rating = "BBB"
            };

            this._itemsList.Add(user1);
            this._itemsList.Add(user2);
            this._itemsList.Add(user3);
        }

        private void InitializePivotGrid()
        {
            FlatDataSource flatDataSource = new FlatDataSource();

            /* allows you to create a virtual dimensions and measures
             * look at userNameMetadata below. 
             * it's not connected with any source property but it
             * requires to have a at least one hierachy descriptor*/
            flatDataSource.DimensionsGenerationMode = DimensionsGenerationMode.Mixed;

            flatDataSource.ItemsSource = this._itemsList;

            // specify default rows/columns/measures
            flatDataSource.Rows = DataSourceBase.GenerateInitialItems("[User Names].[Users]");
            flatDataSource.Columns = DataSourceBase.GenerateInitialItems("[Location].[Locations]");
            flatDataSource.Measures = DataSourceBase.GenerateInitialItems("Rating");
            
            /* metada for specific items source type*/
            CubeMetadata cubeMetadata = 
                new CubeMetadata
                    {
                        DisplayName = "User Data", 
                        DataTypeFullName = typeof(UserData).FullName
                    };

            flatDataSource.CubesSettings.Add(cubeMetadata);

            /* generating default dimensions over 
             * FirstName and LastName is suppressed
             * by property applied attributes 
             * look at the class definition */

            /* define virtual dimension */
            DimensionMetadata userNameMetadata =
                new DimensionMetadata
                    {
                        DisplayName = "User Names",

                        // specify that it is a dimension
                        // because the measure always needs SourcePropertyName
                        // no matter if it is virtual
                        DimensionType = DimensionType.Dimension
                    };

            HierarchyDescriptor<UserData> userNameDescriptor =
                new HierarchyDescriptor<UserData>(ud => ud.FirstName) { HierarchyName = "Users" };
            userNameDescriptor.AddLevel("All Users", "All");
            userNameDescriptor.AddLevel(ud => ud.GetUserFullName(), "User Names");

            /* all virtual dimensions need at least one hierarchy descriptor*/
            userNameMetadata.HierarchyDescriptors.Add(userNameDescriptor);

            // specify metadata about Location
            DimensionMetadata locationMetadata =
                new DimensionMetadata
                    {
                        SourcePropertyName = "Location",
                        DisplayName = "User Locations"
                    };

            // create only measure over Age property
            DimensionMetadata ageMetadata =
                new DimensionMetadata
                    {
                        SourcePropertyName = "Age",
                        DisplayName = "User Age", 
                        DimensionType = DimensionType.Measure
                    };

            // create measure over Rating property using
            // custom aggregator
            DimensionMetadata ratingMetadata =
                new DimensionMetadata
                    {
                        SourcePropertyName = "Rating",
                        DisplayName = "User Rating",
                        DimensionType = DimensionType.Measure,
                        Aggregator = new RatingAggregator()
                    };

            cubeMetadata.DimensionSettings.Add(userNameMetadata);
            cubeMetadata.DimensionSettings.Add(locationMetadata);
            cubeMetadata.DimensionSettings.Add(ageMetadata);
            cubeMetadata.DimensionSettings.Add(ratingMetadata);

            // create a complex hierarchy over Location            
            HierarchyDescriptor<UserData> locationHierarchyDescriptor = 
                new HierarchyDescriptor<UserData>(ud => ud.Location) { HierarchyName = "Locations" };
            locationHierarchyDescriptor.AddLevel("All", "All");
            locationHierarchyDescriptor.AddLevel(ud => ud.GetCountry(), "Countries");
            locationHierarchyDescriptor.AddLevel(ud => ud.GetCity(), "Cities");

            flatDataSource.AddHierarchyDescriptor(locationHierarchyDescriptor);

            this.pivotGrid.DataSource = flatDataSource;
            this.pivotDataSelector.DataSource = flatDataSource;
        }
    }

    /* you can use class methods instead but here we suppose we
     * don't have an access to the class definition*/
    public static class UserDataExtensions
    {
        public static string GetUserFullName(this UserData instance)
        {
            return string.Format("{0} {1}", instance.LastName, instance.FirstName);
        }

        public static string GetCountry(this UserData instance)
        {
            string[] parts = instance.Location.Split('/');
            return parts[1].Trim();
        }

        public static string GetCity(this UserData instance)
        {
            string[] parts = instance.Location.Split('/');
            return parts[0].Trim();
        }
    }

    public class RatingAggregator : Aggregator<string>
    {
        public override IAggregationResult<string, string> Evaluate(IAggregationResult<string, string> oldResult, string value)
        {
            throw new NotImplementedException();
        }

        public override IAggregationResult<string, string> Evaluate(IAggregationResult<string, string> oldResult, IAggregateable aggregateable, System.Collections.IEnumerable items)
        {
            // called for each cell
            // items are cell relevant instances
            string value = null;
            foreach (var item in items)
            {
                string currentValue = (string)aggregateable.GetValue(item);
                if (value != null && currentValue != value)
                {
                    return new SingleResult<string>("Mixed ratings");
                }

                value = currentValue;
            }

            return new SingleResult<string>(value);
        }
    }

}
